#lang racket
(require plot)

(plot (list (axes) ; can also use (axis x y) to specify location to draw axes
            (function sin #:color "Blue" #:label "y = sin x" #:style 'dot)
            (inverse sqr -2 2 #:color "red"  #:label "x = y^2" #:width 2))
      #:x-min (* -2 pi) #:x-max (* 2 pi)
      #:y-min -2 #:y-max 2
      #:title "Sine and Square" 
      #:x-label "X" 
      #:y-label #f
      #:legend-anchor 'bottom-left)   


; define angle constants
(define 0d 0)
(define 90d (/ pi 2))
(define 180d pi)
(define 270d (* 3 (/ pi 2)))
(define 360d (* 2 pi))

(define phi (/ (add1 (sqrt 5)) 2))

; function to compute F(n) (Fibonacci number)
(define (F n)
  (define (f a b cnt)
    (if (= cnt 0) b
        (f (+ a b) a (- cnt 1))))
  (f 1 0 n))

(define (gold-curv θ) (/ 1(expt phi (* θ (/ 2 pi)))))

(define (fib-curv theta)
  (let ([n (+ 3 (floor (/ theta 90d)))]) ; compute n for F(n)
    (/ 1(F n))))

(plot (list
       (function gold-curv #:color "red" #:label "Golden")
       (function fib-curv #:color "blue" #:label "Fibonacci"))
      #:x-min 0 #:x-max (* 6 pi)
      #:y-min 0 #:y-max 1.0
      #:x-label "θ" 
      #:y-label "Curvature"
      #:legend-anchor 'top-right)

; polar plot of golden spiral
(plot (polar (λ (θ) (expt phi (* θ (/ 2 pi))))
             0 (* 4 pi)
             #:x-min -20 #:x-max 50
             #:y-min -40 #:y-max 30
             #:color "red")
      #:title "Golden Spiral"
      #:x-label #f ; supress axis labels
      #:y-label #f)