#lang racket/gui
(require math/matrix)

(define SIZE 5)        ; card rows and columns
(define MARGIN 10)     ; in pixels
(define MSG-HEIGHT 20) ; height of msg label

(define selected-row -1)
(define selected-col -1)
(define show-card #t)

(define card-names
  #("01H" "02H" "03H" "04H" "05H" "06H" "07H" "08H" "09H" "10H" "11H" "12H" "13H"
          "01C" "02C" "03C" "04C" "05C" "06C" "07C" "08C" "09C" "10C" "11C" "12C" "13C"
          "01D" "02D" "03D" "04D" "05D" "06D" "07D" "08D" "09D" "10D" "11D" "12D" "13D"
          "01S" "02S" "03S" "04S" "05S" "06S" "07S" "08S" "09S" "10S" "11S" "12S" "13S"))

(define card-deck 
	(for/vector ([card-name (in-vector card-names)]) 
          (read-bitmap (build-path "Card PNGs" (string-append card-name ".png")))))

; (define card-deck (make-vector 52 0))
; read in card bitmaps
;(for ([i (in-range 52)])
;  (let ([file-name (string-append "Card PNGs\\" (vector-ref card-names i) ".png")])
;    (vector-set! card-deck i (read-bitmap file-name))))

(define (shuffle-deck)
  (for ([i (in-range 52)])
    (let ([j (random 52)]
          [t (vector-ref card-deck i)])
      (vector-set! card-deck i (vector-ref card-deck j))
      (vector-set! card-deck j t))))

(define display-matrix
  (build-matrix SIZE SIZE (lambda (r c) (+ (* r SIZE) c))))

(define (get-card r c)
  (vector-ref card-deck (matrix-ref display-matrix r c)))

(define card-width (send (vector-ref card-deck 0) get-width))
(define card-height (send (vector-ref card-deck 0) get-height))

; window panels --------------------

(define main-frame
  (new frame%
       [label "Pick a Card"]
       [width 550]
       [height 650]))

(define main-panel (new horizontal-panel%
                        [parent main-frame]))

(define control-panel (new vertical-panel%
                           [parent main-panel]
                           [min-width 100]
                           [stretchable-width 100]))

(define canvas
  (new canvas%
       [parent main-panel]
       [min-width 400]
       [paint-callback
        (lambda (canvas dc)
          (send dc set-smoothing 'smoothed)
          (for* ([r (in-range SIZE)]
                 [c (in-range SIZE)])
            (send dc draw-bitmap (get-card r c)
                  (+ MARGIN (* c (+ MARGIN card-width)))
                  (+ MSG-HEIGHT MARGIN (* r (+ MARGIN card-height)))))
          (when show-card
            (let* ([off-x (/ MARGIN 2)]
                   [off-y (+ off-x MSG-HEIGHT)])
              (send dc set-pen "red" 3 'solid)
              (send dc set-brush (new brush% [style 'transparent]))
              (send dc draw-rectangle
                    (+ off-x (* selected-col (+ MARGIN card-width)))
                    (+ off-y (* selected-row (+ MARGIN card-height)))
                    (+ card-width MARGIN) (+ card-height MARGIN))
              (send dc set-pen "black" 2 'solid)))
          )]))

; add controls --------------------

#;(define arrow ; bitmap
    (let* ([image (make-bitmap 50 40)]
           [dc (new bitmap-dc% [bitmap image])]
           [path (new dc-path%)])
      (send dc set-brush (new brush% [color "blue"]))
      (send path move-to  0 10)
      (send path line-to 30 10)
      (send path line-to 30  0)
      (send path line-to 50 20)
      (send path line-to 30 40)
      (send path line-to 30 30)
      (send path line-to  0 30)
      (send path line-to  0 10)
      (send dc draw-path path)
      image))

(define arrow ; bitmap
  (let* ([image (make-bitmap 50 40)]
         [dc (new bitmap-dc% [bitmap image])]
         [path (new dc-path%)])
    (send dc set-brush (new brush% [color "blue"]))
    (send path move-to  0 10)
    (send path
          lines '(
                 (30 . 10)
                 (30 . 0)
                 (50 . 20)
                 (30 . 40)
                 (30 . 30)
                 ( 0 . 30)
                 ( 0 . 10)))
    (send dc draw-path path)
    image))

(define (gen-row-button r)
  (new button%
       [parent control-panel]
       [label arrow]
       [min-width 80]
       [min-height 50]
       [vert-margin (/ (+ MARGIN (- card-height 50)) 2)]
       [callback (lambda (button event)
                   (select-row r))]))

(define msg
  (new message%
       [parent control-panel]
       [min-height MSG-HEIGHT]
       [label "Select again."]))

(for ([i (in-range SIZE)])
  (gen-row-button i))

(new button%
     [parent control-panel]
     [label "Play Again"]
     [callback (lambda (button event) (restart))])

; control logic --------------------

(define (make-swap r1 r2)
  (define (swap-func r c)
    (cond [(= r r1) (if (= c r2) 1 0)]
          [(= r r2) (if (= c r1) 1 0)]
          [(= r c) 1]
          [else 0]))
  (build-matrix SIZE SIZE swap-func))

(define (swap-cols c1 c2)
  (let ([swap-matrix (make-swap c1 c2)])
    (matrix* display-matrix swap-matrix)))
    
(define (first-row-selection r)
  (set! selected-col r)
  (send msg set-label "Select again.")
  (for ([i (in-range SIZE)])
    (let ([j (random SIZE)]
          [t (vector-ref card-deck i)])
      (set! display-matrix (swap-cols i j))))
  (set! display-matrix (matrix-transpose display-matrix))
  (send canvas refresh-now))

(define (show-selection r)
  (send msg set-label "Tada!")
  {set! selected-row r}
  (set! show-card #t)
  (send canvas refresh-now))

(define (select-row r)
  (cond [(< selected-col 0) (first-row-selection r)]
        [(< selected-row 0) (show-selection r)]
        [else (send msg set-label "Restart.")]))

(define (restart)
  (shuffle-deck)
  (send msg set-label "Select a row.")
  (set! show-card #f)
  (set! selected-row -1)
  (set! selected-col -1)
  (send canvas refresh-now))
  
(restart)

(send main-frame show #t)

